@@ -5,6 +5,7 @@
  */
 
 use nalgebra::ComplexField;
+use num_traits::FromPrimitive;
 
 /// Numerically find the derivative of a function at a point.
 ///
@@ -12,11 +13,14 @@ use nalgebra::ComplexField;
 /// step size h, calculate the derivative of f at x using the five-point method.
 ///
 /// Making h too small will lead to round-off error.
-pub fn derivative<N: ComplexField>(
+pub fn derivative<N: ComplexField + FromPrimitive + Copy>(
     f: impl Fn(N::RealField) -> N,
     x: N::RealField,
     h: N::RealField,
-) -> N {
+) -> N
+where
+    <N as ComplexField>::RealField: FromPrimitive + Copy,
+{
     (f(x - h - h) + N::from_f64(8.0).unwrap() * (f(x + h) - f(x - h)) - f(x + h + h))
         / (N::from_f64(12.0).unwrap() * N::from_real(h))
 }
@@ -28,10 +32,13 @@ pub fn derivative<N: ComplexField>(
 /// method.
 ///
 /// Making h too small will lead to round-off error.
-pub fn second_derivative<N: ComplexField>(
+pub fn second_derivative<N: ComplexField + FromPrimitive + Copy>(
     f: impl Fn(N::RealField) -> N,
     x: N::RealField,
     h: N::RealField,
-) -> N {
+) -> N
+where
+    <N as ComplexField>::RealField: FromPrimitive + Copy,
+{
     (f(x - h) - N::from_f64(2.0).unwrap() * f(x) + f(x + h)) / N::from_real(h.powi(2))
 }
